#############################################################################
#
#    OpenERP, Open Source Management Solution    
#    Copyright (C) 2004-2009 TeMPO Consulting (http://www.tempo-consulting.fr)
#    . All Rights Reserved
#    Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#   
#   For more information : please send a message to info@tempo-consulting.fr
#
#   Author : Quentin THEURET (<quentin.theuret@tempo-consulting.fr>)
#
#
##############################################################################

import time
import netsvc
from osv import fields, osv

class chemical_hazard_type(osv.osv):
    '''
        Class to define type of hazard
    '''

    _name = 'chemical.hazard.type'
    _description = 'Type of hazards'

    _columns = {
        'name': fields.char(size=256, string='Name', translate=True,
                            required=True),
    }

    _order = 'name, id'

chemical_hazard_type()


def _get_type(self, cr, uid, context={}):
    result = []
    type_obj = self.pool.get('chemical.hazard.type')
    cols = type_obj.search(cr, uid, [])
    for col in type_obj.read(cr, uid, cols, ['name']):
        result.append( (col['name'][:5], col['name']) )
    result.sort()
    return result


class chemical_hazard_class(osv.osv):
    '''
        Class to define all hazard classes
    '''

    _name = 'chemical.hazard.class'
    _description = 'Class of hazards'

    _columns = {
        'name': fields.char(size=256, string='Name', translate=True,
                            required=True),
        'type': fields.selection(_get_type, string='Type', required=True),
    }

    _order = 'type, name, id'

chemical_hazard_class()


class chemical_hazard_pictogram(osv.osv):
    '''
        Class to represent the pictogram of the hazard
    '''

    _name = 'chemical.hazard.pictogram'
    _description = 'Pictogram of the hazard'

    _columns = {
        'name': fields.char(size=256, string='Name', required=True,
                                translate=True),
        'code': fields.char(size=12, string='Code', translate=True),
        'image': fields.binary(string='Pictogram'),
    }

    _order = 'code, name, id'

chemical_hazard_pictogram()


class chemical_hazard_statement(osv.osv):
    '''
        GHS Hazard Statement class
    '''

    _name = 'chemical.hazard.statement'
    _description = 'Hazard statement'

    _columns = {
        'name': fields.char(size=256, string='Name', required=True,
                                translate=True),
        'code': fields.char(size=12, string='Code', required=True),
    }

    _order = 'code, name, id'

chemical_hazard_statement()


class chemical_precautionary_statement(osv.osv):
    '''
        SGH Precautionary statement class
    '''

    _name = 'chemical.precautionary.statement'
    _description = 'Precautionary statement'

    _columns = {
        'name': fields.char(size=512, string='Name', required=True,
                                translate=True),
        'code': fields.char(size=24, string='Code', required=True),
        'type': fields.selection([('general', 'General'),
                                ('prevention','Prevention'),
                                ('response','Response'),
                                ('storage','Storage'),
                                ('disposal','Disposal')],
                                string='Type', required=True),
        'use_condition': fields.text(string='Conditions for use', 
                                translate=True),
    }

    _order = 'code, name, id'

chemical_precautionary_statement()


class chemical_hazard(osv.osv):
    '''
        Class to represent all chemical hazards
    '''

    _name = 'chemical.hazard'
    _description = 'Chemical hazard'

    _columns = {
        'name': fields.char(size=256, string='Classification', required=True,
                                translate=True),
        'class_id': fields.many2one('chemical.hazard.class',
                                string='Chemical class', required=True),
        'type': fields.related('class_id', 'type', type='selection',
                                selection=_get_type,
                                string='Type', readonly=True),
        'picto_id': fields.many2one('chemical.hazard.pictogram', 
                                        string='Pictogram'),
        'signal_word': fields.selection(
                            [('danger','Danger'),('warning','Warning')],
                            string='Signal word'),
        'hazard_statement_id': fields.many2one('chemical.hazard.statement',
                                        string='Hazard statement'),
        'prevention_ids': fields.many2many('chemical.precautionary.statement',
                                       'hazard_precautionary_prevention_rel',
                                       'hazard_id', 'statement_id',
                                       string='Prevention statement'),
        'response_ids': fields.many2many('chemical.precautionary.statement',
                                       'hazard_precautionary_response_rel',
                                       'hazard_id', 'statement_id',
                                       string='Response statement'),
        'storage_ids': fields.many2many('chemical.precautionary.statement',
                                       'hazard_precautionary_storage_rel',
                                       'hazard_id', 'statement_id',
                                       string='Storage statement'),
        'disposal_ids': fields.many2many('chemical.precautionary.statement',
                                       'hazard_precautionary_disposal_rel',
                                       'hazard_id', 'statement_id',
                                       string='Disposal statement'),
    }

    _order = 'class_id, name, id'

chemical_hazard()


class chemical_additional_hazard(osv.osv):
    '''
        Supplemental hazard information class
    '''

    _name = 'chemical.additional.hazard'
    _description = 'Supplemental hazard information'

    _columns = {
        'name': fields.char(size=256, string='Name', required=True,
                                        translate=True),
        'code': fields.char(size=12, string='Code', required=True),
    }

chemical_additional_hazard()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
