# -*- coding: utf-8 -*-
##############################################################################
#
#    civilengineering module for OpenERP
#    Copyright (C) 2008 Zikzakmedia S.L. (http://zikzakmedia.com)
#       Raimon Esteve <resteve@zikzakmedia.com> All Rights Reserved.
#
#    This file is a part of crm_task
#
#    civilengineering is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    civilengineering is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import time
import tools

from osv import fields,osv
from tools.translate import _

class crm_case(osv.osv):

    _name = "crm.case"
    _inherit = "crm.case"

    _columns = {
        'create_task': fields.boolean('Create a task'),
        'project_task_id': fields.many2one('project.task','Project Task'),
    }

    _defaults = {
        'create_task': lambda *a: 1,
    }


    def case_open(self, cr, uid, ids, *args):
        # Open cases
        # 0. If is not enabled to create work for that case, nothing is done.
        # 1. Look if there is a root project (parent_id = False) that the client is the same as the case partner_id
        # 1.1. If there is this project send an email to the head of case reporting not found what project father associate task
        # 2. See if there is a sub being called 'cases' that his father is the root project
        # 2.1. If not exist, create the subproject
        # 3. Looking to whether there is the task we want to create (depending on case_id and responsible)
        # 3.1. If there is a task created with the values in case the subproject 'case'.

        # Array per guardar les diferents respostes
        res = []
        project_obj = self.pool.get('project.project')
        task_obj = self.pool.get('project.task')

        # Open Cases
        res = super(crm_case, self).case_open(cr, uid, ids, *args)

        cases = self.browse(cr, uid, ids)
        for case in cases:
            # 0
            if not case.create_task:
                continue

            # 1
            search_params = [
                ('parent_id', '=', False),
                ('partner_id', '=', case.partner_id.id)
            ]
            pids = project_obj.search(cr, uid, search_params)
            # 1.1
            if not len(pids):
                msn = _("""Have not found the project to parent company %s (id: %i).
To assign the task in this case you have created the project company or desactive "Create a task" field in Extra info tab.""") % (case.partner_id.name, case.partner_id.id)
                raise osv.except_osv(_('UserError'),msn)
                continue
            # 2.
            year,month,day = case.create_date.split('-')
            project_name = _("Case %s/%s") % (month, year)
            search_params = [
                ('parent_id', '=', pids[0]),
                ('name', '=', project_name)
            ]
            cpids = project_obj.search(cr, uid, search_params)
            project_id = project_obj.browse(cr, uid, pids[0])
            # 2.1
            if not len(cpids):
                vals = {
                    'name': project_name,
                    'parent_id': pids[0],
                    'partner_id': case.partner_id.id,
                    'category_id': project_id['category_id']['id'],
                }
                cpids.append(project_obj.create(cr, uid, vals))
            # 3
            vals = {
                'case_id' : case.id,
                'user_id': case.user_id.id,
                'project_id': cpids[0],
                'partner_id': case.partner_id.id,
                'name': '['+str(case.id)+'] '+case.name,
                'description': case.description,
                'date_start': case.create_date,
                'date_deadline': case.date_deadline,
                'planned_hours': case.planned_cost,
                'priority': str(int(case.priority)-1),
            }
            search_params = [
                ('case_id', '=', case.id),
                ('user_id', '=', case.user_id.id),
                ('project_id', '=', cpids[0]),
                ('partner_id', '=', case.partner_id.id),
            ]
            tids = task_obj.search(cr, uid, search_params)
            if not len(tids):
                project_task_id = task_obj.create(cr, uid, vals)
                tids.append(project_task_id)
                self.write(cr, uid, case.id, {'project_task_id':project_task_id})
            else:
                vals['state'] = 'open'
                task_obj.write(cr, uid, tids, vals)

            # msg = """New task #%i: %s (id: %i) in subproject %s to %s""" % (case.id, case.name, tids[0], project_name, case.partner_id.name)
            # print msg
        return res
    
    def case_close(self, cr, uid, ids, *args):
        # Sobreescrivim el mètode close per tal que quan es tanqui una tasca
        # tanqui totes les tasques que trobi relacionades amb aquest cas
        # Array per guardar les diferents respostes
        res = []
        # project_obj = self.pool.get('project.project')
        task_obj = self.pool.get('project.task')
        cases = self.browse(cr, uid, ids)

        for case in cases:
                task = self.pool.get('project.task').search(cr, uid, [('case_id','=',case.id)])
                if task:
                        search_params = [
                                ('case_id', '=', case.id),
                        ]
                        tids = task_obj.search(cr, uid, search_params)
                        if tids:
                                task_obj.write(cr, uid, tids, {'state': 'done'})

                res.append(super(crm_case, self).case_close(cr, uid, ids, *args))

        # Retornem True si totes les subcrides han anat bé
        # o False si alguna no ha anat bé
        return reduce(lambda x,y: x and y, res)

    def crm_task_work(self, cr, uid, data, *args):
        taskwork_obj = self.pool.get('project.task.work')
        task_ids = self.pool.get('project.task').search(cr, uid, [('case_id','=',data['ids'][0])])

        if task_ids:
            vals = {
                'name': data['form']['description'],
                'date': time.strftime('%Y-%m-%d %H:%M:%S'),
                'task_id': task_ids[0],
                'hours': data['form']['hours'],
                'user_id': uid,
            }

            taskwork_obj.create(cr, uid, vals)

            return task_ids
        else:
            raise osv.except_osv(_('UserError'), _("""Not avaible any project task related to this partner. Reopen this case for create a task."""))
            return False

crm_case()
