# -*- coding: utf-8 -*-
##############################################################################
#
#    civilengineering module for OpenERP
#    Copyright (C) 2008 Zikzakmedia S.L. (http://zikzakmedia.com)
#       Raimon Esteve <resteve@zikzakmedia.com> All Rights Reserved.
#
#    This file is a part of crm_task
#
#    civilengineering is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    civilengineering is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import wizard
import netsvc
import pooler
import time
import locale
from tools.translate import _

class task_work_caselog(wizard.interface):
    task_form = """<?xml version="1.0"?>
    <form string="Task Work">
        <separator colspan="4" string="Insert time to task work:" />
        <field name="description" />
        <field name="hours" widget="float_time"/>
    </form>
    """

    task_fields = {
        'description': {
            'string': 'Description',
            'type': 'char',
            'size': 128,
            'required': True
        },
        'hours': {
            'string': 'Time Spent',
            'type': 'float',
        },
    }

    def _crmcase(self, cr, uid, data, context):
        pool = pooler.get_pool(cr.dbname)
        pool.get('crm.case').case_log(cr, uid, data['ids'], context)
        return {}

    def _check(self, cr, uid, data, context):
        pool = pooler.get_pool(cr.dbname)
        case_obj = pool.get('crm.case')
        case = case_obj.browse(cr, uid, data['id'], context)
        if case['create_task'] == True:
            action = 'crmtask'
        else:
            action = 'end'
        return action

    def _taskWork(self, cr, uid, data, context):
        pool = pooler.get_pool(cr.dbname)
        task_ids = pool.get('crm.case').crm_task_work(cr, uid, data, context)

        if task_ids:
            value = {
                'domain': [
                    ('id', '=', task_ids[0]),
                    ],
                'name': 'Project Task',
                'view_type': 'form',
                'view_mode': 'tree,form',
                'res_model': 'project.task',
                'type': 'ir.actions.act_window',
            }
            return value

    states = {
        'init': {
            'actions': [_crmcase],
            'result': {
                'type': 'choice',
                'next_state': _check,
            }
        },
        'crmtask' : {
            'actions' : [],
            'result' : {'type' : 'form',
                    'arch' : task_form,
                    'fields' : task_fields,
                    'state' : [('end', 'Cancel'),('crm', 'Register Task Work') ]}
        },
        'crm' : {
            'actions' : [],
            'result' : {'type' : 'action',
                    'action' : _taskWork,
                    'state' : 'end'}
        },
        'end': {
            'actions': [],
            'result': {'type': 'state', 'state': 'end'},
        },
    }
task_work_caselog("crm.caselog")
