#-*- coding:utf-8 -*-

##############################################################################
#
#    Copyright (C) 2010 De Smet Nicolas (<http://ndesmet.be>).
#    All Rights Reserved
#    
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

''' Features:
 - Send only email once per email
 - Easily send again a mailing to a partner
 - Test can be made on mailing
 - Easily select target, by category, by partner or by partner address
 - Multi-languages: send the email body and attachment in the language of the 
   partner
 - Partner can refuse mailing
 - Attachment can be reused in another mailing without having to be uploaded 
   again
 - Support html email with plain text, to avoid being taken as threat by 
   SpamAssassin
'''


from osv import osv
from osv import fields

class MailingAddress(osv.osv):
    '''
    Add the field refuse_mailing to contact
    '''
    _name = 'res.partner.address'
    _inherit='res.partner.address'
    
    _columns = {
        'refuse_mailing':fields.boolean('Refuse mailing', 
            help="Checked means that this contact refuse to receive mailing."),
    }
MailingAddress()

class MailingAttachment(osv.osv):
    '''
    Mailing attachment model
    '''
    _name = 'mailing.attachment'
    _description = 'Mailing attachment model'
    
    # Many2many syntax need to be simplified...
    # ie: 
    # 'lang_ids': fields.many2many('res.lang', 'Languages'),
    # Other things are low level feature that user don't need to take care
    _columns = {
        'name':fields.char('Attachment name', size=64, required=True),
        'data':fields.binary('File'),
        'datas_fname': fields.char('Filename',size=64),
        'lang_ids': fields.many2many('res.lang', 'lang_mailing_attachment_rel', 
                                     'lang_id', 'mailing_attachment_id',
                                     'Languages'),
    }
MailingAttachment()

class Mailing(osv.osv):
    '''
    Mailing model
    '''
    _name = 'mailing'
    _description = 'Mailing model'
    
    # Many2many syntax really need to be simplified...
    _columns = {
        'name':fields.char('Mailing name', size=64, required=True),
        'email_from':fields.char('Email from', size=64, required=True),
        'content_title':fields.char('Email title', size=200, translate=True),
        'content_text':fields.text('Content text', translate=True),
        'content_html':fields.text('Content html', translate=True),
        'content_embed_image':fields.boolean('Embed image', help='Embed all image external images targeted by HTML content in the email.'),
        'content_attachment_ids':fields.many2many('mailing.attachment', 
                'mailing_mailing_attachment_rel', 'mailing_attachement_id', 
                'mailing_id', 'Attachments'),
        'target_category_ids':fields.many2many('res.partner.category', 
                'mailing_partner_category_rel', 'partner_category_id', 
                'mailing_id', 'Target partner category'),
        'target_partner_ids':fields.many2many('res.partner', 
                'mailing_partner_rel', 'partner_id', 'mailing_id', 
                'Target partner'),
        'target_partner_address_ids':fields.many2many('res.partner.address', 
                'mailing_partner_address_rel', 'partner_address_id', 
                'mailing_id', 'Target partner address'),
        'target_partner_default': fields.boolean('Default'),
        'target_partner_invoice': fields.boolean('Invoice'),
        'target_partner_delivery': fields.boolean('Delivery'),
        'target_partner_contact': fields.boolean('Contact'),
        'target_partner_other': fields.boolean('Other'),
        'sent_to_ids':fields.many2many('res.partner.address', 
                'mailing_sent_to_ids_rel', 'partner_address_id', 'mailing_id', 
                'Partner address who already received the email'),
    }
Mailing()

