#-*- coding:utf-8 -*-

##############################################################################
#
#    Yet Another Send Mail Module
#
#    Copyright (C) 2010 De Smet Nicolas (<http://ndesmet.be>).
#    All Rights Reserved
#
#    Based on PowerEmail from Sharoon Thomas 
#    (that's need OpenObject 5.2 to run because of mako dependency)
#    Based on OpenObject server tools misc.py
#    (that's lacking txt and html body, html body alone lead to spam detected
#     mail by SpamAssassin)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import re
import smtplib
import urllib
from email.MIMEText import MIMEText
from email.MIMEBase import MIMEBase
from email.MIMEMultipart import MIMEMultipart
from email.MIMEImage import MIMEImage
from email.Header import Header
from email.Utils import formatdate, COMMASPACE
from email import Encoders
import tools
from tools.config import config
import netsvc

email_regex = None

def is_valid_email(email):
    # Filter email against RFC2821 
    # available at http://www.ietf.org/rfc/rfc2821.txt
    # Input: email
    # Output: True if email is valid
    if not isinstance(email, basestring): return False
    global email_regex
    if email_regex is None:
        regex = '^[a-z0-9._%-]+[a-z0-9_%-]+@[a-z0-9._%-]+.[a-z]{2,6}$'
        email_regex = re.compile(regex, re.IGNORECASE)
    return not email_regex.match(email) is None

def keep_valid_emails(emails):
    # Filter emails against RFC2821 
    # available at http://www.ietf.org/rfc/rfc2821.txt
    # Input: List of emails
    # Output: List of emails without invalid one
    valid_emails = []
    for email in emails:
        if is_valid_email(email): valid_emails.append(email)
    return valid_emails
   
def get_ids_from_dict(addresses={}):
    result = {'all':[]}
    keys = ['To', 'CC', 'BCC']
    for key in keys:
        ids_as_list = addresses.get(key, [])
        while u'' in ids_as_list:
            ids_as_list.remove(u'')
        result[key] = ids_as_list
        result['all'].extend(keep_valid_emails(ids_as_list))
    return result

class mail_image_inliner(object):
    '''Replace all "src" in "<img>" tag with an image ID and add a the image
    in mail attachment.
    
    See more information at: http://code.activestate.com/recipes/473810/'''
    BASE_IMG_NAME = 'image'
    def __init__(self, html):
        self.img_urls = []
        patern = r'(?i)(<(?:img.*src|.*background)\s*=\s*("|\'))([a-z0-9.:/\\\-_%]*)\2' 
        self.new_html = re.sub(patern, self.replace, html)
        
    def replace(self, matchobj):
        # Replace url by a cid:imageid
        tag_begin = matchobj.group(1)
        tag_end = matchobj.group(2)
        url = matchobj.group(3)
        try:
            urllib.urlopen(url)
        except:
            return matchobj.group(0)
        if url not in self.img_urls:
            index = len(self.img_urls)
            self.img_urls.append(url)
        else:
            index = self.img_urls.index(url)
        return tag_begin + 'cid:' + self.BASE_IMG_NAME + str(index) + tag_end
    
    def get_new_html(self):
        return self.new_html
    
    def get_container(self, mail_container):
        containerMIME = MIMEMultipart('related')
        containerMIME.attach(mail_container)
        for id in range(len(self.img_urls)):
            url = self.img_urls[id]
            img_id = self.BASE_IMG_NAME + str(id)
            img_data = urllib.urlopen(url).read()
            filename = url.split('/').pop()
            msgImage = MIMEImage(img_data)
            msgImage.add_header('Content-ID', img_id)
            msgImage.add_header('Content-Disposition', 'inline; filename="%s"' % filename)
            containerMIME.attach(msgImage)
        return containerMIME            


def send_mail(email_from, addresses = {}, subject='', body={}, attachments={}, 
              ssl=False, reply_to=False, debug=False, embed_image=False):
    '''
    Send an email directly through SMTP server in config file.

    addresses = {
                'To':['toto@company.com'], 
                'CC':['joe@company.com', 'jack@company.com'], 
                'BCC':['spy@company.com']
                },
    subject = 'My mail subject',
    body = {
            'text': 'My email content',
            'html': '<html><body><h2>Title</h2>My email content</body></html>',
            },
    attachments = [
                  ('filename.pdf', 'File_Content'),
                  ],
    ssl = True
    '''
    if not ssl:
        ssl = config.get('smtp_ssl', False)

    if not email_from and not config['email_from']:
        raise Exception("No Email sender by default, see config file")
    
    if body.get('text', False):
        temp_body_text = body.get('text', '')
        l = len(temp_body_text.replace(' ', '').replace('\r', '').\
                replace('\n', ''))
        if l == 0:
            body['text'] = u'No Mail Message'
    # Attach parts into message container.
    # According to RFC 2046, the last part of a multipart message, in this case
    # the HTML message, is best and preferred.
    body_text = body.get('text', u'No Mail Message')
    body_text = tools.ustr(body_text)
    body_text_mime = MIMEText(body_text.encode("utf-8"), _charset='UTF-8') 
    msg = MIMEMultipart()
    if body.get('html', False):
        # to avoid showing text and html at the same time we should use a 
        # sub-container with subtype 'alternative'
        body_multi = MIMEMultipart('alternative')
        body_multi.attach(body_text_mime)
        html_body = tools.ustr(body['html'])        
        if embed_image:
            mii = mail_image_inliner(html_body) 
            html_body = mii.get_new_html()
            msg = mii.get_container(body_multi)
        else:
            msg.attach(body_multi)
        body_multi.attach(MIMEText(html_body.encode("utf-8"), _subtype='html', 
                            _charset='UTF-8'))
    else:
        msg.attach(body_text_mime)
    #Now add attachments if any
    for (filename, data) in attachments:
        part = MIMEBase('application', "octet-stream")
        part.set_payload(data)
        part.add_header('Content-Disposition', 'attachment', filename=filename)
        Encoders.encode_base64(part)
        msg.attach(part)
    
    msg['Subject'] = Header(tools.ustr(subject), 'utf-8')
    msg['From'] = email_from
    del msg['Reply-To']
    if reply_to:
        msg['Reply-To'] = reply_to
    else:
        msg['Reply-To'] = msg['From']
        
    addresses_l = get_ids_from_dict(addresses)
    if addresses_l['To']:   
        msg['To'] = COMMASPACE.join(addresses_l['To'])
    if addresses_l['CC']:
        msg['Cc'] = COMMASPACE.join(addresses_l['CC'])
    msg['Date'] = formatdate(localtime=True)
    
    class WriteToLogger(object):
        def __init__(self):
            self.logger = netsvc.Logger()

        def write(self, s):
            self.logger.notifyChannel('email_send', netsvc.LOG_DEBUG, s)
    try:
        oldstderr = smtplib.stderr
        s = smtplib.SMTP()
        
        try:
            # in case of debug, the messages are printed to stderr.
            if debug:
                smtplib.stderr = WriteToLogger()

            s.set_debuglevel(int(bool(debug)))  # 0 or 1
            
            s.connect(config['smtp_server'], config['smtp_port'])
            if ssl:
                s.ehlo()
                s.starttls()
                s.ehlo()

            if config['smtp_user'] or config['smtp_password']:
                s.login(config['smtp_user'], config['smtp_password'])

            s.sendmail(email_from, addresses_l['all'], msg.as_string())

        finally:
            s.quit()
            if debug:
                smtplib.stderr = oldstderr

    except Exception, e:
        netsvc.Logger().notifyChannel('email_send', netsvc.LOG_ERROR, e)
        return False
    
    return True