#-*- coding:utf-8 -*-

##############################################################################
#
#    Copyright (C) 2010 De Smet Nicolas (<http://ndesmet.be>).
#    All Rights Reserved
#    
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import wizard
import pooler
import base64
import cPickle
from mailing_core import is_valid_email, send_mail

def _get_partner_addresses(cr, uid, context, pool, partner_ids, addr_types):
    # Retrieve all partner addresses
    partner_address_ids = []
                    
    # Apply address type and refuse mailing filter
    filter = [('partner_id', 'in', partner_ids), ('refuse_mailing', '=', False)]
    
    # Only filter by addr_types if one address type is selected
    if addr_types: filter += [('type', 'in', addr_types)]
    partner_address_ids += pool.get('res.partner.address').search(cr, uid, 
                                                        filter, context=context)
    return partner_address_ids

def get_already_sent_ids(mailing):
    # Check against already sent to avoid sending a email twice to the 
    # same partner_address (contact)
    already_sent_ids = []
    for already_sent in mailing.sent_to_ids:
        already_sent_ids += [already_sent.id]
    return already_sent_ids

all_address_types = ['default', 'invoice', 'delivery', 'contact', 'other']

def send_mailing(pool, mailing, partner_address_ids, force = False):
    
    return {'sent':sent_ids, 'no_sent': not_send_ids}

def _send_mailing(cr, uid, mailing_obj, mailing, emails_to, context):
    nb_email_sent = 0
    for (lang, emails) in emails_to.iteritems():
        if not emails: continue
        attachments = []
        # Only add attachment if attachment.lang is not set 
        # or if attachment.lang contains current lang
        for attachment in mailing.content_attachment_ids:
            if len(attachment.lang_ids) == 0 or \
                reduce(lambda x, y: x or (y.code == lang), 
                       attachment.lang_ids, False):
                attachments += [(attachment.datas_fname, 
                                 base64.decodestring(attachment.data))]
        emails = list(emails)
        # Retrieve the translation for content text and title
        context_lang = context.copy()
        context_lang['lang'] = lang
        content = mailing_obj.read(cr, uid, mailing.id, 
            ['content_title', 'content_text', 'content_html', 
             'content_embed_image'], 
            context=context_lang)
        if send_mail(mailing.email_from, {
                'To':['undisclosed-recipients:;'], 'BCC': emails},
                 content['content_title'],
                 {'text':content['content_text'], 
                  'html':content['content_html']}, 
                 attachments, embed_image=content['content_embed_image']):
            nb_email_sent += len(emails)
    return nb_email_sent

def _get_emails_to(partner_addresses):
    # Retrieve valid emails and check for only send one mail per address
    emails_to = {}
    all_emails_to = set()
    partner_address_ids_sent = set()
    for partner_address in partner_addresses:
        if partner_address.email and is_valid_email(partner_address.email) and \
          not partner_address.email.lower() in all_emails_to:               
            all_emails_to.add(partner_address.email.lower())
            emails_to.setdefault(partner_address.partner_id.lang, 
                            set()).add(partner_address.email.lower())
            partner_address_ids_sent.add(partner_address.id)
    return (emails_to, partner_address_ids_sent)
    
def _do_presend(cr, uid, pool, mailing_obj, mailing, partner_address_ids, 
                force=False):
    # Do all actions before mailing
    already_sent_ids = get_already_sent_ids(mailing)
    if force:
        partner_address_ids_to_send = partner_address_ids
    else:
        partner_address_ids_to_send = \
                list(set(partner_address_ids).difference(set(already_sent_ids)))
    partner_addresses = pool.get('res.partner.address').browse(cr, uid, 
                                                    partner_address_ids_to_send)
    (emails_to, partner_address_sent_ids) = _get_emails_to(partner_addresses)
    
    # Add partner_address to sent_to_ids M2M list
    new_ids = list(partner_address_sent_ids.union(set(already_sent_ids)))
    mailing_obj.write(cr, uid, mailing.id, 
                              {'sent_to_ids': [(6, 0, new_ids)]})
    partner_address_not_sent_ids = \
        list(set(partner_address_ids).difference(set(partner_address_sent_ids)))
    sent_details = {'sent_to_ids': list(partner_address_sent_ids),
                    'not_sent_to_ids': partner_address_not_sent_ids}
    return emails_to, sent_details  

def _send_mailing_by_itself(self, cr, uid, data, context):
    pool = pooler.get_pool(cr.dbname)
    res_lang_obj = pool.get('res.lang') 
    to_email = ('to_email' in data['form']) and data['form']['to_email']
    to_language = False
    if 'lang_id' in data['form']:
        lang_id = data['form']['lang_id']
        if lang_id is not False:
            result = res_lang_obj.read(cr, uid, lang_id, ['code'])
            to_language = result['code']
    mailing_ids = data['ids']
    nb_email_sent, sent_details = _do_mailing(cr, uid, mailing_ids, context, 
                                              to_email, to_language=to_language)
    return {'nb_email_sent': nb_email_sent}
    
def _get_partner_ids_from_mailing(cr, uid, context, pool, mailing):
    partner_ids = []
    # Retrieve all partners
    for category in mailing.target_category_ids:
        partner_ids += pool.get('res.partner').search(cr, uid, 
                                [('category_id','child_of', [category.id])], 
                                context=context)
    for partner in mailing.target_partner_ids:
        partner_ids += [partner.id]
        partner_ids = list(set(partner_ids)) # Remove duplicate

    # Retrieve all partner addresses
    addr_types = []
    for addr_type in all_address_types:
        if getattr(mailing, 'target_partner_' + addr_type):
            addr_types += [addr_type]
    partner_address_ids = _get_partner_addresses(cr, uid, context, pool, 
                                                 partner_ids, addr_types)
    for partner_address in mailing.target_partner_address_ids:
        partner_address_ids += [partner_address.id]
    return partner_address_ids
                    
def _do_mailing(cr, uid, mailing_ids, context, to_email = False, 
                all_partner_address_ids = {}, force=False, to_language=False):
    """ Send emails
    """
    pool = pooler.get_pool(cr.dbname)
    mailing_obj = pool.get('mailing') 
    nb_email_sent = 0
    sent_details = {}
    for mailing in mailing_obj.browse(cr, uid, mailing_ids):
        if to_email:
            if not to_language:
                to_language = context['lang'] 
            emails_to = {to_language:[to_email]}
        else:
            if all_partner_address_ids:
                partner_address_ids = all_partner_address_ids
            else:
                partner_address_ids = _get_partner_ids_from_mailing(cr, uid, 
                                                        context, pool, mailing) 
            
            emails_to, sent_details[mailing.id] = _do_presend(cr, uid, pool, 
                               mailing_obj, mailing,partner_address_ids, force)
        
        nb_email_sent += _send_mailing(cr, uid, mailing_obj, mailing, emails_to, 
                                       context)
    return nb_email_sent, sent_details


email_sent_form = '''<?xml version="1.0" encoding="utf-8"?>
<form string="Mailing result">
    <field name="nb_email_sent"/>
</form>'''
email_sent_fields = {
    'nb_email_sent': {'string': 'Number of email sent', 'type': 'integer', 
                      'readonly': True},
}
email_sent_state = [('end', 'Ok')]
email_sent_result = {'type': 'form', 'arch': email_sent_form, 
                       'fields': email_sent_fields, 'state': email_sent_state}

class MailingTest(wizard.interface):
    '''
    Wizard that send mailing to only one address, for testing purpose
    '''
    form = '''<?xml version="1.0"?>
    <form string="Send a mailing test">
        <field name="to_email"/>
        <newline />
        <field name="lang_id"/>
    </form>'''

    fields = {
        'to_email': {'string': 'Email', 'type': 'char', 'size': 60},
        'lang_id': {'string': 'Langue', 'type': 'many2one', 
                    'relation':'res.lang', 'domain':[('translatable', '=', True)]},
    }

    def _get_defaults(self, cr, user, data, context):
        data['form']['to_email'] = ''
        return data['form'] 
   
    states = {
        'init': {
            'actions': [_get_defaults],
            'result': {'type': 'form', 'arch': form, 'fields': fields, 
                       'state': [('end', 'Cancel'), ('send', 'Send')]},
        },
        'send': {
            'actions': [_send_mailing_by_itself],
            'result': email_sent_result,
        },
    }
MailingTest('mailing.test')

class MailingSend(wizard.interface):
    '''
    Wizard that send mailing
    '''
    form = '''<?xml version="1.0"?>
    <form string="Send a mailing">
        <label string="Are you sure ?"/>
    </form>'''
    
    states = {
        'init': {
            'result': {'type': 'form', 'arch': form, 'fields':{}, 
                       'state': (('end', 'Cancel'), ('send', 'Send'))},
        },
       'send': {
            'actions': [_send_mailing_by_itself],
            'result': email_sent_result,
        },
    }
MailingSend('mailing.send')

class MailingToOne(wizard.interface):
    '''
    Wizard that send mailing to selected partner only
    '''
    
    form_already_sent = '''<?xml version="1.0"?>
<form string="Send a mailing">
<separator string="Those partner addresses already received the mailing" 
  colspan="4"/> 
<field name="partner_address_ids" nolabel="1" colspan="4"/>
</form>
    '''

    fields = {
              'mailing_ids': {'string':'Mailings', 'type':'many2many', 
                              'relation':'mailing'},
              }
    
    forced_view = '''<?xml version="1.0"?>
<form string="Send a mailing">
<field name="nb_email_sent"/>
<separator colspan="4"/>
<field name='mailing_id' readonly="1"/>
<separator string="Those partner addresses already received the mailing" 
  colspan="4"/>
<field name='not_sent_to_ids' colspan="4" nolabel="1">

</field>
</form>'''
    '''  <tree string="Partner address">
    <field name="partner_id" />
    <field name="name" />
    <field name="email" />
  </tree>'''
    forced_fields = {
        'mailing_id' : {'string':'Mailing', 'type':'many2one', 
                        'relation':'mailing'},
        'not_sent_to_ids': {
                'string':"Partner Addresses that didn't receive this mailing", 
                'type':'many2many', 'relation':'res.partner.address'},
        'nb_email_sent': {'string': 'Number of email sent', 'type': 'integer', 
                          'readonly': True},
                     }
    forced_state = (('end', 'Cancel'), ('check_to_force', 'Send without them'), 
                    ('do_forced', 'Send to them again'))
    forced_result = {'type': 'form', 'arch': forced_view, 
                     'fields': forced_fields, 'state': forced_state}
    
    all_sent_fields = {}
    
    def _set_form_and_fields(self, is_partner):
        partner_add = ''
        if is_partner:
            partner_add = '''
<group colspan="4" col="10">
    <separator 
    string="Partner address type  (Selecting 'none' acts as 'all selected'.)" 
    colspan="10" />'''
            for a_type in all_address_types:
                partner_add += '<field name="target_partner_'+ a_type + '"/>'
                self.states['init']['result']['fields']\
                    ['target_partner_'+ a_type] = {'string':a_type, 
                                                   'type':'boolean'}
            partner_add += '</group>'
        self.states['init']['result']['arch'] =  '''<?xml version="1.0"?>
<form string="Send a mailing">''' + partner_add + '''
    <separator string="Select the mailings" colspan="4"/> 
    <field name="mailing_ids" nolabel="1" colspan="4"/>
</form>'''
    
    def _get_defaults(self, cr, user, data, context):
        self._set_form_and_fields(data['model'] == 'res.partner')
        return data['form']
    
    def _return_data(self, data, all_sent_details, state):
        for (mailing_id, sent_details) in all_sent_details.iteritems(): 
            if (sent_details['not_sent_to_ids']):
                all_sent_details.pop(mailing_id)
                # Avoid compatibility problem with XML-RPC => pickle
                data['form']['all_sent_details'] = \
                                                cPickle.dumps(all_sent_details)
                data['form']['mailing_id'] = mailing_id
                data['form']['not_sent_to_ids'] = \
                                                sent_details['not_sent_to_ids']
                # Change view and fields in a dynamic way
                self.states[state]['result']['arch'] = self.forced_view
                self.states[state]['result']['fields'] = self.forced_fields
                self.states[state]['result']['state'] = self.forced_state
                #= self.forced_result
                return data['form']
        self.states[state]['result']['arch'] = email_sent_form
        self.states[state]['result']['fields'] = email_sent_fields
        self.states[state]['result']['state'] = email_sent_state
        return data['form']

    def _check(self, cr, uid, data, context):
        pool = pooler.get_pool(cr.dbname)
        # retrieve partner_address_ids
        partner_address_ids = []
        if data['model'] == 'res.partner':
            addr_types = []
            for addr_type in all_address_types:
                if data['form'].get('target_partner_' + addr_type, False):
                    addr_types += [addr_type]
            partner_address_ids = _get_partner_addresses(cr, uid, context, pool, 
                                                         data['ids'],
                                                         addr_types)
        elif data['model'] == 'res.partner.address':
            partner_address_ids = data['ids']
        
        mailing_ids = data['form']['mailing_ids'][0][2]
        nb_email_sent, all_sent_details = _do_mailing(cr, uid, mailing_ids, 
                        context, all_partner_address_ids=partner_address_ids)       
        data['form']['nb_email_sent'] = nb_email_sent
        return self._return_data(data, all_sent_details, 'check')
    
    def _do_forced(self, cr, uid, data, context):
        partner_address_ids = data['form']['not_sent_to_ids'][0][2]
        _do_mailing(cr, uid, [data['form']['mailing_id']], context, 
                    all_partner_address_ids=partner_address_ids, 
                    force=True)
        data['form']['nb_email_sent'] = len(partner_address_ids)
        all_sent_details = cPickle.loads(data['form']['all_sent_details'])
        return self._return_data(data, all_sent_details, 'do_forced')
    
    def _check_to_force(self, cr, uid, data, context):
        data['form']['nb_email_sent'] = 0
        all_sent_details = cPickle.loads(data['form']['all_sent_details'])
        return self._return_data(data, all_sent_details, 'check_to_force')
    
    states = {
        'init': {
            'actions': [_get_defaults],
            'result': {'type': 'form', 'fields': fields, 
                       'state': (('end', 'Cancel'), ('check', 'Send'))},
        },
       'check': { 
                 'actions': [_check],
                 'result': email_sent_result,
                 },
       'do_forced': { 
                     'actions': [_do_forced],
                 'result': email_sent_result,
                 },
       'check_to_force': { 
                          'actions': [_check_to_force],
                 'result': email_sent_result,
                 },
    }
MailingToOne('mailing.send2one')