# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import fields
from osv import osv
import ir
from tools import config

import netsvc
import time
from mx import DateTime
from tools.translate import _
import math


def name_serial(x, y, z, p, f, d):

     lote = '_'
     if (f == 'quadrangular'):
         lote = (str(x) + 'x' + str(y) + 'x' + str(z) + '_' + str(p))
     elif (f == 'cylindrical'):
         lote = (str(d) + 'x' + str(z) + '_' + str(p))

     return lote

def compute_w(cr, uid, factor1, factor2, factor3, factor4, size_x, size_y, size_z, shape, density, diameter):
       resu = {}
       
       weight = 0
       if shape == 'quadrangular':
           weight = ((size_x * size_y * size_z) * density * factor2 * (math.pow(factor3, 3))) / (factor1 * (math.pow(factor4, 3))) 
           diameter = 0
       elif shape == 'cylindrical':
               radius = diameter / 2.0
               weight = ((math.pow(radius, 2) * math.pi * size_z) * density * factor2 * (math.pow(factor3, 3))) / (factor1 * (math.pow(factor4, 3))) 
               size_x = 0
               size_y = 0
       else:
               weight = 0
               size_x = 0
               size_y = 0               
               size_z = 0

               
       v = { 'weight':weight,
             'size_x':size_x,
             'size_y':size_y,
             'size_z':size_z,
             'diameter':diameter,
             'density':density,
             }
       return v  




class product_product(osv.osv):
    _inherit = 'product.product'
    _columns = {
        'size_x': fields.float('Width'),
        'size_y': fields.float('Length'),
        'size_z': fields.float('Thickness'),
        'density': fields.float('Density', help='Density unit= (uom of weight/(uom of size)^3)'),
        'shape' : fields.selection([('quadrangular', 'Quadrangular'), ('cylindrical', 'Cylindrical'), ('other', 'Other')], 'Shape', help="Weight Formula for Quadrangular = width*length*thickness*density \n Weight Formula for Cylindrical= (diameter/2)²*pi*thickness*density", required=True),
        'diameter' : fields.float('Diameter'),
        'purchase_price': fields.selection((('weight', 'Weight'), ('units', 'Units')), 'Purchase price in', help="United of measure for purchase operations", required=True),
        'uom_d_size' : fields.many2one('product.uom', 'Size density cubic Uom', help="Default united of measure density used for operations size to cube", required=True),
        'uom_d_weight': fields.many2one('product.uom', 'Weight density Uom', help="Default united of measure density used for operations weight", required=True),
        'uom_s_size' : fields.many2one('product.uom', 'Size Uom', help="Default united of measure used for operations size", required=True),
        
    }
    _defaults = {
        'diameter': lambda * a: 0.0,
        'shape': lambda * a: 'other',
        'size_x': lambda * a: 0.0,
        'size_y': lambda * a: 0.0,
        'size_z': lambda * a: 0.0,
        'density': lambda * a: 0.0,
        'purchase_price': lambda * a: 'units',
        'uom_d_size' : lambda * a: 1,
        'uom_d_weight' : lambda * a: 1,
        'uom_s_size' : lambda * a: 1,
        }
    
    def compute_weight(self, cr, uid, id, uom_d_weight, uom_id, uom_d_size, uom_s_size, size_x, size_y, size_z, shape, density, diameter):
        value = {}

        if (not uom_id) or (not uom_d_weight) or(not uom_d_size) or (not uom_s_size):
            return 0
        if (shape):
           factor1 = self.pool.get('product.uom').browse(cr, uid, uom_d_weight, context='').factor
           factor2 = self.pool.get('product.uom').browse(cr, uid, uom_id, context='').factor
           factor3 = self.pool.get('product.uom').browse(cr, uid, uom_d_size, context='').factor
           factor4 = self.pool.get('product.uom').browse(cr, uid, uom_s_size, context='').factor                      
           return {'value': compute_w(cr, uid, factor1, factor2, factor3, factor4, size_x, size_y, size_z, shape, density, diameter)}
   

product_product()        

class stock_production_lot(osv.osv):
    _name = 'stock.production.lot'
    _inherit = 'stock.production.lot'
    _columns = {
        'size_x': fields.float('Width'),
        'size_y': fields.float('Length'),
        'size_z': fields.float('Thickness'),
        'shape' : fields.selection([('quadrangular', 'Quadrangular'), ('cylindrical', 'Cylindrical'), ('other', 'Other')], 'Shape', required=True),
        'diameter' : fields.float('Diameter'),
        'weight': fields.float('Weight'),
        'density': fields.float('Density'),
    }
    _defaults = {
        'diameter': lambda * a: 0.0,
        'shape': lambda * a: 'other',
        'size_x': lambda * a: 0.0,
        'size_y': lambda * a: 0.0,
        'size_z': lambda * a: 0.0,
        'weight': lambda * a: 0.0,
    }
    
    def generate_serial(self, cr, uid, ids, context={}):
       radio = 0
       for w in self.browse(cr, uid, ids):
           v = {
                'name' : name_serial(w.size_x, w.size_y, w.size_z, w.weight, w.shape, w.diameter)
                }
           w.write(v)

    def compute_weight(self, cr, uid, id, product_id, size_x, size_y, size_z, shape, density, diameter):
       if product_id:
           product = self.pool.get('product.product').browse(cr, uid, product_id, context='')
           factor1 = product.uom_d_weight.factor
           factor2 = product.uom_id.factor
           factor3 = product.uom_d_size.factor
           factor4 = product.uom_s_size.factor           
           return {'value': compute_w(cr, uid, factor1, factor2, factor3, factor4, size_x, size_y, size_z, shape, density, diameter)}
    
stock_production_lot()    

class mrp_production_product_line(osv.osv):
    _name = 'mrp.production.product.line'    
    _inherit = 'mrp.production.product.line'
    _columns = { 
        'size_x': fields.float('Width'),
        'size_y': fields.float('Lenght'),
        'size_z': fields.float('Thickness'),
        'density': fields.float('Density'),
        'shape' : fields.selection([('quadrangular', 'Quadrangular'), ('cylindrical', 'Cylindrical'), ('other', 'Other')], 'Shape', required=True),
        'diameter' : fields.float('Diameter'),
        'weight': fields.float('Weight')
                       
        }
    _defaults = {
        'shape': lambda * a: 'other',
         }


    def onchange_product_id(self, cr, uid, ids, product_id, context={}):
        if product_id:
            w = self.pool.get('product.product').browse(cr, uid, product_id, context)
            factor1 = w.uom_d_weight.factor
            factor2 = w.uom_id.factor
            factor3 = w.uom_d_size.factor
            factor4 = w.uom_s_size.factor                       
            v = {
                'product_uom':w.uom_id.id,
                'product_uos':w.uos_id and w.uos_id.id or w.uom_id.id,
                'size_x':w.size_x,
                'size_y':w.size_y,
                'size_z':w.size_z,
                'density':w.density,
                'shape':w.shape,
                'diameter':w.diameter,
                'weight':compute_w(cr, uid, factor1, factor2, factor3, factor4, w.size_x, w.size_y, w.size_z, w.shape, w.density, w.diameter)['weight']                                                                               
            }
            return {'value': v}
        return {}

    def compute_weight(self, cr, uid, id, product_id, size_x, size_y, size_z, shape, density, diameter):
       if product_id:
           product = self.pool.get('product.product').browse(cr, uid, product_id, context='')
           factor1 = product.uom_d_weight.factor
           factor2 = product.uom_id.factor
           factor3 = product.uom_d_size.factor
           factor4 = product.uom_s_size.factor           
           return {'value': compute_w(cr, uid, factor1, factor2, factor3, factor4, size_x, size_y, size_z, shape, density, diameter)}
    
    
    
mrp_production_product_line()
    
class mrp_production(osv.osv):
    _name = 'mrp.production'
    _description = 'Production'
    _inherit = 'mrp.production'    
    _columns = { 
        'product_line_origin': fields.many2one('mrp.production.product.line', 'Linea de produccion original'),
    }

    def action_compute(self, cr, uid, ids, properties=[]):
        results = []

        result = super(mrp_production, self).action_compute(cr, uid, ids, properties)
        
        for production in self.browse(cr, uid, ids):
            i = 0
            bom_id = production.bom_id.id
            for a in self.pool.get('mrp.production.product.line').search(cr, uid, [('production_id', '=', production.id)]):
                production_line = self.pool.get('mrp.production.product.line').browse(cr, uid, a)
                w = False
                w2 = False
                if not (production.product_line_origin):
                    p = self.pool.get('mrp.bom').browse(cr, uid, bom_id)
                    w = self.pool.get('mrp.bom').browse(cr, uid, p.bom_lines[i].id)
                    if w:
                        v = {                        
                            'product_uom':w.product_uom.id,
                            'product_uos':w.product_uos and w.product_uos.id or w.product_uom.id,
                            'size_x':w.size_x,
                            'size_y':w.size_y,
                            'size_z':w.size_z,
                            'density':w.density,
                            'shape':w.shape,
                            'diameter':w.diameter,
                            'weight':w.weight                                                                               
                            }
                else:
                    origen = production.origin[1:]

                    w2 = self.pool.get('mrp.production.product.line').browse(cr, uid, production.product_line_origin.id)
                    v = {
                        'size_x':w2.size_x,
                        'size_y':w2.size_y,
                        'size_z':w2.size_z,
                        'density':w2.density,
                        'shape':w2.shape,
                        'diameter':w2.diameter,
                        'weight':w2.weight                                                                               
                    }
                if (w or w2):
                    production_line.write(v)
                i = i + 1
                
        return len(results)       

    def action_confirm(self, cr, uid, ids):
   
        picking_id = super(mrp_production, self).action_confirm(cr, uid, ids)
        for production in self.browse(cr, uid, ids):     
            stock_move_id = False
            lote_id = False
            routing_loc = None
            pick_type = 'internal'
            address_id = False            
            if production.bom_id.routing_id and production.bom_id.routing_id.location_id:
                routing_loc = production.bom_id.routing_id.location_id
                if routing_loc.usage <> 'internal':
                    pick_type = 'out'
                address_id = routing_loc.address_id and routing_loc.address_id.id or False
                routing_loc = routing_loc.id

            
            if (production.product_line_origin and production.product_line_origin.id):
                product_line_orig = self.pool.get('mrp.production.product.line').browse(cr, uid, production.product_line_origin.id)

                name_seriale = name_serial(product_line_orig.size_x, product_line_orig.size_y, product_line_orig.size_z, product_line_orig.weight, product_line_orig.shape, product_line_orig.diameter)
                l = self.pool.get('stock.production.lot').search(cr, uid, [('product_id', '=', product_line_orig.product_id.id), ('name', '=', name_seriale)])
                if not l:
                        datalote = {
                            'name':name_seriale,
                            'product_id': product_line_orig.product_id.id,
                            'date' : production.date_planned
                            }
                        lote_id = self.pool.get('stock.production.lot').create(cr, uid, datalote)
                else:
                    lote_id = l[0]
            source = production.product_id.product_tmpl_id.property_stock_production.id   
            stock_move_id = self.pool.get('stock.move').search(cr, uid, [('name', '=', 'PROD:' + production.name),
                                                                       ('product_id', '=', production.product_id.id),
                                                                       ('location_id', '=', source),
                                                                       ('location_dest_id', '=', production.location_dest_id.id),
                                                                       ('move_dest_id', '=', production.move_prod_id.id)])
            if (stock_move_id and stock_move_id[0]):
                self.pool.get('stock.move').write(cr, uid, stock_move_id[0], {'prodlot_id': lote_id or False})                                        

            products = {}
           
            for line in production.product_lines:
                mrp_producurement_id = None
                if products.has_key(line.product_id.id):
                    products[line.product_id.id] = products[line.product_id.id] + 1
                else: 
                    products[line.product_id.id] = 0                     

                stock_move_id2 = False
                stock_move_id3 = False               
                name_seriale = name_serial(line.size_x, line.size_y, line.size_z, line.weight, line.shape, line.diameter)
                l = self.pool.get('stock.production.lot').search(cr, uid, [('product_id', '=', line.product_id.id), ('name', '=', name_seriale)])
                if not l:
                        datalote = {
                            'name':name_seriale,
                            'product_id': line.product_id.id,
                            'date' : production.date_planned,
                            'size_x' : line.size_x,
                            'size_y' : line.size_y,
                            'size_z' : line.size_z,
                            'shape' : line.shape,
                            'diameter' : line.diameter,
                            'density' : line.density,
                            'weight' : line.weight,
                            
                            }
                        lote_id2 = self.pool.get('stock.production.lot').create(cr, uid, datalote)
                else:
                    lote_id2 = l[0]

                if line.product_id.type in ('product', 'consu'):                
                    stock_move_id2 = self.pool.get('stock.move').search(cr, uid, [('name', '=', 'PROD:' + production.name),
                                                                       ('product_id', '=', line.product_id.id),
                                                                       ('location_id', '=', routing_loc or production.location_src_id.id),
                                                                       ('location_dest_id', '=', source),
                                                                       ('move_dest_id', '=', stock_move_id),
                                                                       ('product_qty', '=', line.product_qty)])
                    if (stock_move_id2 and stock_move_id2[products[line.product_id.id]]):
                        self.pool.get('stock.move').write(cr, uid, stock_move_id2[products[line.product_id.id]], {'prodlot_id': lote_id2 or False})


            
                    stock_move_id3 = self.pool.get('stock.move').search(cr, uid, [('name', '=', 'PROD:' + production.name),
                                                                       ('product_id', '=', line.product_id.id),
                                                                       ('location_id', '=', production.location_src_id.id),
                                                                       ('location_dest_id', '=', routing_loc or production.location_src_id.id),
                                                                       ('move_dest_id', '=', stock_move_id2[products[line.product_id.id]])])
                    if (stock_move_id3 and stock_move_id3[0]):
                        self.pool.get('stock.move').write(cr, uid, stock_move_id3[0], {'prodlot_id': lote_id2 or False})
                if (stock_move_id3 and stock_move_id3[0]):
                    mrp_procurement_id = self.pool.get('mrp.procurement').search(cr, uid, [('move_id', '=', stock_move_id3[0])])
                    self.pool.get('mrp.procurement').write(cr, uid, mrp_procurement_id, {'product_line_origin' : line.id})
                    origin = (production.origin or '') + ':' + production.name
                    mrp_production_id = self.pool.get('mrp.production').search(cr, uid, [('origin', '=', origin), ('product_id', '=', line.product_id.id)]) 
                    mrp_production_id.sort()

                    data = {
                            'size_x' : line.size_x,
                            'size_y' : line.size_y,
                            'size_z' : line.size_z,
                            'shape' : line.shape,
                            'diameter' : line.diameter,
                            'density' : line.density,
                            'weight' : line.weight,
                            }

                    if mrp_production_id:
                        self.pool.get('mrp.production').write(cr, uid, mrp_production_id[products[line.product_id.id]], {'product_line_origin' : line.id})
                        production_product_id = self.pool.get('mrp.production.product.line').search(cr, uid, [('production_id', '=', mrp_production_id[products[line.product_id.id]])])
                        self.pool.get('mrp.production.product.line').write(cr, uid, production_product_id, data)
                    else:
                        if production.product_line_origin:
                            data = {
                            'size_x' : production.product_line_origin.size_x,
                            'size_y' : production.product_line_origin.size_y,
                            'size_z' : production.product_line_origin.size_z,
                            'shape' : production.product_line_origin.shape,
                            'diameter' : production.product_line_origin.diameter,
                            'density' : production.product_line_origin.density,
                            'weight' : production.product_line_origin.weight,
                            }
                            
                        for purchase in self.pool.get('purchase.order.line').search(cr, uid, [('production_id', '=', production.id)]):
                                self.pool.get('purchase.order.line').write(cr, uid, purchase, data)                        
                        
        return picking_id

mrp_production() 
 
 
 
 
 
class mrp_procurement(osv.osv):
    '''
    Sobrescribimos la función para crear movimientos en borrador
    '''
    _name = 'mrp.procurement'
    _inherit = 'mrp.procurement'
    _columns = { 
        'product_line_origin': fields.many2one('mrp.production.product.line', 'Product line origin'),
    }
    def action_produce_assign_product(self, cr, uid, ids, context={}):
        produce_id = False
        company = self.pool.get('res.users').browse(cr, uid, uid, context).company_id
        for procurement in self.browse(cr, uid, ids):
            res_id = procurement.move_id.id
            loc_id = procurement.location_id.id
            newdate = DateTime.strptime(procurement.date_planned, '%Y-%m-%d %H:%M:%S') - DateTime.RelativeDateTime(days=procurement.product_id.product_tmpl_id.produce_delay or 0.0)
            newdate = newdate - DateTime.RelativeDateTime(days=company.manufacturing_lead)
            produce_id = self.pool.get('mrp.production').create(cr, uid, {
                'origin': procurement.origin,
                'product_id': procurement.product_id.id,
                'product_qty': procurement.product_qty,
                'product_uom': procurement.product_uom.id,
                'product_uos_qty': procurement.product_uos and procurement.product_uos_qty or False,
                'product_uos': procurement.product_uos and procurement.product_uos.id or False,
                'location_src_id': procurement.location_id.id,
                'location_dest_id': procurement.location_id.id,
                'bom_id': procurement.bom_id and procurement.bom_id.id or False,
                'date_planned': newdate.strftime('%Y-%m-%d %H:%M:%S'),
                'move_prod_id': res_id,
                'product_line_origin' : procurement.product_line_origin and procurement.product_line_origin.id or False,
            })
            self.write(cr, uid, [procurement.id], {'state':'running'})
            bom_result = self.pool.get('mrp.production').action_compute(cr, uid,
                    [produce_id], properties=[x.id for x in procurement.property_ids])
            
#            wf_service = netsvc.LocalService("workflow")
#            wf_service.trg_validate(uid, 'mrp.production', produce_id, 'button_confirm', cr)
            
            self.pool.get('stock.move').write(cr, uid, [res_id],
                    {'location_id':procurement.location_id.id})
        return produce_id
    
    
    def action_po_assign(self, cr, uid, ids, context={}):
        line = {}
        purchase_id = False
        purchase_id = super(mrp_procurement, self).action_po_assign(cr, uid, ids, context)
        for procurement in self.browse(cr, uid, ids):
            production_name = procurement.origin
            production_name = production_name.split(':')
            for prod_name in production_name:
               production_id = self.pool.get('mrp.production').search(cr, uid, [('name', '=', prod_name)])
               if production_id:
                   break
            purchase_order_line_id = self.pool.get('purchase.order.line').search(cr, uid, [('order_id', '=', purchase_id)])
            for purchase_orderline in purchase_order_line_id:
                self.pool.get('purchase.order.line').write(cr, uid, purchase_orderline, {'production_id':production_id[0]})
                
        return purchase_id

    
    
mrp_procurement() 


class purchase_order(osv.osv):
    _name = "purchase.order"  
    _inherit = "purchase.order"    
    
    def _amount_all(self, cr, uid, ids, field_name, arg, context):
        res = {}
        res = super(purchase_order, self)._amount_all(cr, uid, ids, field_name, arg, context) 
        cur_obj = self.pool.get('res.currency')
        for order in self.browse(cr, uid, ids):
            res[order.id] = {
                'amount_untaxed': 0.0,
                'amount_tax': 0.0,
                'amount_total': 0.0,
            }
            val = val1 = 0.0
            cur = order.pricelist_id.currency_id
            for line in order.order_line:

                if line.purchase_price == 'weight':
                    product_qty = line.weight * line.product_qty
                else:
                    product_qty = line.product_qty
                for c in self.pool.get('account.tax').compute(cr, uid, line.taxes_id, line.price_unit, product_qty, order.partner_address_id.id, line.product_id, order.partner_id):
                    val += c['amount']
                val1 += line.price_subtotal
            res[order.id]['amount_tax'] = cur_obj.round(cr, uid, cur, val)
            res[order.id]['amount_untaxed'] = cur_obj.round(cr, uid, cur, val1)
            res[order.id]['amount_total'] = res[order.id]['amount_untaxed'] + res[order.id]['amount_tax']
        return res

    def _get_order(self, cr, uid, ids, context={}):
        result = {}
        return super(purchase_order, self)._get_order(cr, uid, ids,context) 
#        for line in self.pool.get('purchase.order.line').browse(cr, uid, ids, context=context):
#            result[line.order_id.id] = True
#        return result.keys()
 
    _columns = {
        'amount_untaxed': fields.function(_amount_all, method=True, string='Untaxed Amount',
            store={
                'purchase.order.line': (_get_order, None, 10),
            }, multi="sums"),
        'amount_tax': fields.function(_amount_all, method=True, string='Taxes',
            store={
                'purchase.order.line': (_get_order, None, 10),
            }, multi="sums"),
        'amount_total': fields.function(_amount_all, method=True, string='Total',
            store={
                'purchase.order.line': (_get_order, None, 10),
            }, multi="sums"),
}
    
    
    def action_picking_create(self, cr, uid, ids, *args):
        picking_id = False
        picking_id = super(purchase_order, self).action_picking_create(cr, uid, ids, *args)

        for order in self.browse(cr, uid, ids):
            for order_line in order.order_line:

                lote_id = False
              
                production_line_orig = self.pool.get('mrp.production.product.line').search(cr, uid,
                    [('production_id', '=', order_line.production_id.id),
                     ('product_id', '=', order_line.product_id.id),
                     ('size_x', '=', order_line.size_x),
                     ('size_y', '=', order_line.size_y),
                     ('shape', '=', order_line.shape)])
                if production_line_orig and production_line_orig[0] :
                    product_line_orig = self.pool.get('mrp.production.product.line').browse(cr, uid, production_line_orig[0])
                    name_seriale = name_serial(product_line_orig.size_x, product_line_orig.size_y, product_line_orig.size_z, product_line_orig.weight, product_line_orig.shape, product_line_orig.diameter)
                    l = self.pool.get('stock.production.lot').search(cr, uid, [('product_id', '=', product_line_orig.product_id.id), ('name', '=', name_seriale)])
                    if not l:
                            datalote = {
                                'name':name_seriale,
                                'product_id': product_line_orig.product_id.id,
                                'date' : production.date_planned,
                                'size_x' : product_line_orig.size_x,
                                'size_y' : product_line_orig.size_y,
                                'size_z' : product_line_orig.size_z,
                                'shape' : product_line_orig.shape,
                                'diameter' : product_line_orig.diameter,
                                'density' : product_line_orig.density,
                                'weight' : product_line_orig.weight                                                                
                                }
                            lote_id = self.pool.get('stock.production.lot').create(cr, uid, datalote)
                    else:
                        lote_id = l[0]

                if not order_line.product_id:
                    continue
                if order_line.product_id.product_tmpl_id.type in ('product', 'consu'):
                    stock_move_id = self.pool.get('stock.move').search(cr, uid, [('purchase_line_id', '=', order_line.id)])
                    if stock_move_id:
                        self.pool.get('stock.move').write(cr, uid, stock_move_id, {'prodlot_id':lote_id})                                                                                    
        return picking_id


purchase_order()


class purchase_order_line(osv.osv):
    _name = "purchase.order.line"
    _inherit = "purchase.order.line"
    
    def _amount_line(self, cr, uid, ids, prop, unknow_none, unknow_dict):
        res = {}
        res = super(purchase_order_line, self)._amount_line(cr, uid, ids, prop, unknow_none, unknow_dict)        
        cur_obj = self.pool.get('res.currency')
        for line in self.browse(cr, uid, ids):
            cur = line.order_id.pricelist_id.currency_id
            if line.purchase_price == 'weight':
                product_qty = line.weight * line.product_qty
            else:
                product_qty = line.product_qty
            res[line.id] = cur_obj.round(cr, uid, cur, line.price_unit * product_qty)
        return res
    

    
    def product_id_change(self, cr, uid, ids, pricelist, product, qty, uom,
            partner_id, date_order=False, fiscal_position=False):
        result = super(purchase_order_line, self).product_id_change(cr, uid, ids, pricelist, product, qty, uom,
            partner_id, date_order, fiscal_position)
        if not product:
            return result
        for product_obj in self.pool.get('product.product').browse(cr, uid, [product]):
            result['value']['purchase_price'] = product_obj.purchase_price
            result['value']['shape'] = product_obj.shape
            result['value']['size_x'] = product_obj.size_x
            result['value']['size_y'] = product_obj.size_y
            result['value']['size_z'] = product_obj.size_z
            result['value']['density'] = product_obj.density
            result['value']['diameter'] = product_obj.diameter
            result['value']['weight'] = compute_w(cr, uid, product_obj.uom_d_weight.factor, product_obj.uom_id.factor, product_obj.uom_d_size.factor, product_obj.uom_s_size.factor, product_obj.size_x, product_obj.size_y, product_obj.size_z, product_obj.shape, product_obj.density, product_obj.diameter)['weight']        

        return result
    
    _columns = {
        'production_id': fields.many2one('mrp.production', 'Produccion', readonly=True),
        'size_x': fields.float('Width',),
        'size_y': fields.float('Length',),
        'size_z': fields.float('Thickness',),
        'density': fields.float('Density',),
        'shape' : fields.selection([('quadrangular', 'Quadrangular'), ('cylindrical', 'Cylindrical'), ('other', 'Other')], 'Shape', required=True,),
        'diameter' : fields.float('Diameter',),
        'weight': fields.float('Weight',),
        'purchase_price': fields.selection((('weight', 'Weight'), ('units', 'Units')), 'Price in', required=True),
        'price_subtotal': fields.function(_amount_line, method=True, string='Subtotal'),
    }
    _defaults = {
        'shape': lambda * a: 'other',
        'purchase_price': lambda * a: 'units',
         }
    def compute_weight(self, cr, uid, id, product_id, size_x, size_y, size_z, shape, density, diameter):
       if product_id:
           product = self.pool.get('product.product').browse(cr, uid, product_id, context='')
           factor1 = product.uom_d_weight.factor
           factor2 = product.uom_id.factor
           factor3 = product.uom_d_size.factor
           factor4 = product.uom_s_size.factor           
           return {'value': compute_w(cr, uid, factor1, factor2, factor3, factor4, size_x, size_y, size_z, shape, density, diameter)}
       
purchase_order_line()

class mrp_bom(osv.osv):
    _name = 'mrp.bom'
    _description = 'Bill of Material'
    _inherit = 'mrp.bom'    
    _columns = { 
        'size_x': fields.float('Width'),
        'size_y': fields.float('Length'),
        'size_z': fields.float('Thickness'),
        'density': fields.float('Density'),
        'shape' : fields.selection([('quadrangular', 'Quadrangular'), ('cylindrical', 'Cylindrical'), ('other', 'Other')], 'Shape', required=True),
        'diameter' : fields.float('Diameter'),
        'weight': fields.float('Weight'),
        }

        
    def compute_weight(self, cr, uid, id, product_id, size_x, size_y, size_z, shape, density, diameter):
       if product_id:
           product = self.pool.get('product.product').browse(cr, uid, product_id, context='')
           factor1 = product.uom_d_weight.factor
           factor2 = product.uom_id.factor
           factor3 = product.uom_d_size.factor  
           factor4 = product.uom_s_size.factor           
           return {'value': compute_w(cr, uid, factor1, factor2, factor3, factor4, size_x, size_y, size_z, shape, density, diameter)}
   
    def onchange_product_id(self, cr, uid, ids, product_id, name, id, context={}):
        if product_id:
            w = self.pool.get('product.product').browse(cr, uid, [product_id], context)[0]
            factor1 = w.uom_d_weight.factor
            factor2 = w.uom_id.factor
            factor3 = w.uom_d_size.factor
            factor4 = w.uom_s_size.factor            
            v = {
                'product_uom':w.uom_id.id,
                'product_uos':w.uos_id and w.uos_id.id or w.uom_id.id,
                'size_x':w.size_x,
                'size_y':w.size_y,
                'size_z':w.size_z,
                'density':w.density,
                'shape':w.shape,
                'diameter':w.diameter,
                'weight': compute_w(cr, uid, factor1, factor2, factor3, factor4, w.size_x, w.size_y, w.size_z, w.shape, w.density, w.diameter)['weight'],
                'name' : w.name,
            }
            if id :
                v['bom_id'] = id
            return {'value': v}
        return {}
           
mrp_bom()

